%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Copyright (C) 2004 Adam Spiro and Aharon Bar-Hillel
%    An implementation of a spike sorting algorithm according to the paper:
%    "Spike Sorting: Bayesian Clustering of Non-Stationary Data" 
%    by Aharon Bar-Hillel, Adam Spiro and Eran Stark 
%    (http://www.cs.huji.ac.il/~aharonbh/papers/SpikeSort_NIPS04.ps)
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% this function runs over  a spike train and gathers mixture solutions in
% time windows.

% input : data -the data
%               GP - Gathering parameters structure. created by SetAllParams.
%               time - the time occurences of spikes. 
% output : PSolutions -a cell array of size ( GParams.WinNum, GParams.SolNum)
%                   containing the i-th mixture suggesion of time window t at (t,i)
%              PLL - the log likelihood of the solutions. size and
%                   indexing as those of solution
%              N - a vector (1,WinNum) of spike counts in the windows

function [ PSolutions, PLL, N ]=gatherSolutions3(data,time,GP)

% initialization
global GParams;
GParams=GP;

% computing basic solution for each time frame.

disp('computing basic solutions in each time frame seperately');
if GParams.UseConstWindowSizeFlag
    N=GParams.SIW;
end

for i=1:GParams.maxK-GParams.minK+1
    c=0;
    K=GParams.minK+i-1;
    disp(sprintf('Computing solutions of K=%d',K));
    for j=1:GParams.WinNum
        disp(sprintf('Time window %d',j));
        
        % create temporary data set to work on
        if GParams.UseConstWindowSizeFlag
            tData=data(c+1:c+N(j),:);
            c=c+N(j);
        else
            Inds=find(time>(j-1)*GParams.TimeWin & time<=j*GParams.TimeWin); % find the indexes of the i-th time window
            tData=data(Inds,:);
            N(j)=length(tData);     % record the data sizes
        end
        chunks=-ones(N(j),1);
        
        % gather SolNum solutions for (time frame i, number of clusters K)
        for m=1:GParams.SolNum
            [ Solutions{j,(i-1)*GParams.SolNum+m}, tmpLL, Groups, z, stam ,exit_flags ]=...
                mixture_of_gaussians5(tData, K, chunks, [],GParams.EMParams);
            LL(j,(i-1)*GParams.SolNum+m)=tmpLL(end);
        end
    end
end

save SolutionsBeforePrune Solutions LL N
OrigSolutions=Solutions;
OrigLL=LL;
[PSolutions, PLL]=PruneAndSort(Solutions, LL,GParams,data,N,1,1,1); % prune and sort - from the beginning

% mix solutions
for repeat=1:GParams.MixingRepeatition
    disp(sprintf('Mixing Solutions iteration %d',repeat));
    Ssize = size(PSolutions,2);
    [Solutions, LL] = mixingSolutions(PSolutions,PLL,data,time,GParams,N);
    [PSolutions, PLL]=PruneAndSort(Solutions, LL,GParams,data,N,1,1,Ssize+1); % prune the new solutions and sort
end

[PSolutions, PLL]=PruneAndSort(PSolutions, PLL,GParams,data,N,0,1,1); % Now only sort without prun

% In case we prunned all the solutions, take the most probable one
for i=1:size(Solutions,1)
    a=fmap('isempty',PSolutions(i,:));
    if(sum([a{:}]==0)==0)
        PSolutions(i,1)=OrigSolutions(i,1);
        PLL(i,1)=OrigLL(i,1);
    end
end