%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Copyright (C) 2004 Adam Spiro and Aharon Bar-Hillel
%    An implementation of a spike sorting algorithm according to the paper:
%    "Spike Sorting: Bayesian Clustering of Non-Stationary Data" 
%    by Aharon Bar-Hillel, Adam Spiro and Eran Stark 
%    (http://www.cs.huji.ac.il/~aharonbh/papers/SpikeSort_NIPS04.ps)
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% This function creates a unified labels vector for all the time windows,
% based on the path computed by the HMM. It amount to a clusure of the label
% correspondences between the choosen mixtures in each time frame.
% Input :     Data - the data points
%                   Solutions - the solutions cell
%                   N - the window spike counts
%                   path - the optimal clustering path computed
%                   Correspondences - the clustering correspondences cell
%                   GP - the GParams staruct used in the data gathering
% Output :  Labels -  a unified labels vector of size length(data).
%           RecordGraph - a data structure that holds the flow of the
%                         clusters. For each time frame i and each cluster j
%                         RecordGraph{i,j} contains:
%                        .timeOfCreation - if it's a new cluster (also from split or merge) 
%                                          it gets the current time window.
%                                          Otherwise, gets the parent's value.
%                        .parents - a vector [i1 i2 ...] which contains the
%                                   cluster's parents (it can have multiple in case of
%                                   a merge, or 0 if it's a new cluster)
%                        .children - a vector of the children in the next time window.
%                                    it has multiple values in case of a split.
%
%

function [Labels, RecordGraph]=prepareUnitedLabels(Data,Solutions,N,path,Correspondences,tGP);

global nextColor;
global GP;
GP=tGP;

c=N(1)+1;
WinNum=size(Solutions,1);
clear Lables;
Labels(1:N(1)) =  confidentAss(Solutions{1,path(1)}, Data(1:N(1),1:GP.DataDim),...
                    GP.EMParams,GP.ConfidenceMin);
usedColorsCur = zeros(1,GP.MaxNumOfColors);
usedColorsPrev = zeros(1,GP.MaxNumOfColors);

% initialize RecordGraph with the first mixture
for i=1:size(Solutions{1,path(1)},1)
    RecordGraph{1,i}.timeOfCreation=0;
    RecordGraph{1,i}.parents=[0];
    RecordGraph{1,i}.color=i;
    usedColorsCur(i)=1; % the colors appearing in this time window
end
nextColor=i+1;

for i=2:WinNum
    usedColorsPrev = usedColorsCur; % save the prev colors
    usedColorsCur = zeros(1,GP.MaxNumOfColors);
    tmp=confidentAss(Solutions{i,path(i)}, Data(c:c+N(i)-1,1:GP.DataDim),GP.EMParams,GP.ConfidenceMin);
    trans=Correspondences{i-1,path(i-1),path(i)};
    trans=trans(:)';
    M1=size(Solutions{i-1,path(i-1)},1);
    M2=size(Solutions{i,path(i)},1);
    
    for j=1:length(trans)
        a=find(trans==j); % a contains the indices of the clusters which should be the same
        switch (length(a))
            case 0 % Do nothing
                
            case 2 % Meaning this is a 1-to-1 correspondence
                RecordGraph{i,a(2)-M1}.timeOfCreation=RecordGraph{i-1,a(1)}.timeOfCreation; % same time as the parent
                RecordGraph{i,a(2)-M1}.parents=a(1); % has one parent
                RecordGraph{i-1,a(1)}.children_ind=a(2)-M1; % the parent has this one child
                RecordGraph{i,a(2)-M1}.color=RecordGraph{i-1,a(1)}.color; % same color as its parent
                usedColorsCur(RecordGraph{i,a(2)-M1}.color) = 1; % update the current used colors
                inds=find(tmp==a(2)-M1);
                Labels(inds+c-1) = ones(1,length(inds))*RecordGraph{i,a(2)-M1}.color;
                RecordGraph{i-1,a(1)}.children_color=RecordGraph{i,a(2)-M1}.color; % the parent has this one child
            case 1 % It's either a new culster or one that disappeared
                if (a(1)>M1) % it's a new cluster
                    RecordGraph{i,a(1)-M1}.timeOfCreation=i; % created right now
                    RecordGraph{i,a(1)-M1}.parents=0; % has no parents
                    RecordGraph{i,a(1)-M1}.color = getNewColor;   %getNewColor(usedColorsCur,usedColorsPrev);
                    usedColorsCur(RecordGraph{i,a(1)-M1}.color) = 1; % update the current used colors
                    inds=find(tmp==a(1)-M1);
                    Labels(inds+c-1) = ones(1,length(inds))*RecordGraph{i,a(1)-M1}.color;
                else % it's a cluster that disappeared
                    RecordGraph{i-1,a(1)}.children_ind=0; % the previous has no children
                    RecordGraph{i-1,a(1)}.children_color=0;
                end
                
            otherwise % It's either a split or a merge
                if (a(2) > M1) % it's a split
                    if (size(a([2:end]),2)==2 & ...
                            a(1)>1 & a(2)-M1==1) % there are 2 children where one is the background
                        
                        RecordGraph{i,a(2)-M1}.timeOfCreation=RecordGraph{i-1,a(1)}.timeOfCreation; % same time as the parent
                        RecordGraph{i,a(3)-M1}.timeOfCreation=RecordGraph{i-1,a(1)}.timeOfCreation; % same time as the parent
                        RecordGraph{i,a(2)-M1}.parents=a(1); % has one parent
                        RecordGraph{i,a(3)-M1}.parents=a(1); % has one parent
                        RecordGraph{i-1,a(1)}.children_ind=a(2:3)-M1; % the parent has two children
                        RecordGraph{i,a(3)-M1}.color=RecordGraph{i-1,a(1)}.color; % same color as its parent
                        RecordGraph{i,a(2)-M1}.color=1; %the background
                        usedColorsCur(RecordGraph{i,a(3)-M1}.color) = 1; % update the current used colors
                        inds=find(tmp==a(2)-M1);
                        Labels(inds+c-1) = ones(1,length(inds))*RecordGraph{i,a(2)-M1}.color;                    
                        inds=find(tmp==a(3)-M1);
                        Labels(inds+c-1) = ones(1,length(inds))*RecordGraph{i,a(3)-M1}.color;                    
                        RecordGraph{i-1,a(1)}.children_color=[RecordGraph{i,a(2)-M1}.color RecordGraph{i,a(3)-M1}.color]; % the parent has two children
                    else
                        RecordGraph{i-1,a(1)}.children_ind = a(2:end)-M1; % these are all children of split source
                        RecordGraph{i-1,a(1)}.children_color = []; 
                        
                        for p=2:length(a)
                            RecordGraph{i,a(p)-M1}.timeOfCreation=i; % created right now
                            RecordGraph{i,a(p)-M1}.parents=a(1); % all have the same father
                            if a(p)==M1+1      % the splitted model is the background
                                RecordGraph{i,a(p)-M1}.color=1;
                            else
                                RecordGraph{i,a(p)-M1}.color =getNewColor;    % getNewColor(usedColorsCur,usedColorsPrev); % get new color
                                usedColorsCur(RecordGraph{i,a(p)-M1}.color) = 1; % update the current used colors
                            end
                            inds=find(tmp==a(p)-M1);
                            Labels(inds+c-1) = ones(1,length(inds))*RecordGraph{i,a(p)-M1}.color;
                            RecordGraph{i-1,a(1)}.children_color = [RecordGraph{i-1,a(1)}.children_color RecordGraph{i,a(p)-M1}.color];
                        end
                    end
                else % it's a merge
                    if (size(a([2:end]),2)==2 & ...
                        a(1)==1 & a(end)-M1>1) % one cluster and the background are merged to another cluster
                        RecordGraph{i,a(end)-M1}.timeOfCreation=RecordGraph{i-1,a(2)}.timeOfCreation;
                        RecordGraph{i,a(end)-M1}.parents=a(2);    
                        RecordGraph{i-1,a(1)}.children_ind = a(end)-M1;
                        RecordGraph{i-1,a(2)}.children_ind = a(end)-M1;
                        RecordGraph{i,a(end)-M1}.color = RecordGraph{i-1,a(2)}.color;
                        RecordGraph{i-1,a(1)}.children_color = RecordGraph{i,a(end)-M1}.color;
                        RecordGraph{i-1,a(2)}.children_color = RecordGraph{i,a(end)-M1}.color;
                        
                    else
                        RecordGraph{i,a(end)-M1}.timeOfCreation=i; % created right now
                        RecordGraph{i,a(end)-M1}.parents=a(1:(end-1)); % has multiple fathers (all the merged clusters)
                        if a(end)==(M1+1)         % it's a merge with the background
                            RecordGraph{i,a(end)-M1}.color=1;
                        else                % it's aregular merge
                            RecordGraph{i,a(end)-M1}.color =getNewColor;  % getNewColor(usedColorsCur,usedColorsPrev); % get new color
                            usedColorsCur(RecordGraph{i,a(end)-M1}.color) = 1; % update the current used colors
                        end
                        for p=1:length(a)-1
                            RecordGraph{i-1,a(p)}.children_ind = a(end)-M1; % this merged cluster is the child of all it's fathers
                            RecordGraph{i-1,a(p)}.children_color = RecordGraph{i,a(end)-M1}.color;
                        end

                    end
                    inds=find(tmp==a(end)-M1);
                    Labels(inds+c-1) = ones(1,length(inds))*RecordGraph{i,a(end)-M1}.color;
                end
                
            end
    end
    inds=find(tmp==1); %the background
    Labels(inds+c-1)=1;
    c=c+N(i);
end

% This function returns a color (index) which doesn't appear in the current
% or previous colors.

function [col] = getNewColor
global nextColor;
global GP;
col=nextColor;
nextColor=nextColor+1;
%if nextColor>GP.MaxNumOfColors
 %   disp('Error: Not enough colors. Please increase GP.MaxNumOfColors!');
%    nextColor
%    return;
GP.MaxNumOfColors=nextColor;
%end